package member_center

import (
	"applet/app/db"
	"applet/app/e"
	md2 "applet/app/md/member_center"
	svc "applet/app/svc/member_center"
	"applet/app/utils"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"errors"
	"github.com/gin-gonic/gin"
	"time"
)

// GetTagList
// @Summary      制度中心-会员中心-标签管理(获取)
// @Tags         会员中心
// @Description  标签管理(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    limit    query	 int 	true  "每页大小"
// @Param    page    query	 int 	true  "页数"
// @Success      200   {object}  md.GetTagListResp    	"具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/memberCenter/tagManagement/getTagList [get]
func GetTagList(c *gin.Context) {
	page := c.DefaultQuery("page", "1")
	limit := c.DefaultQuery("limit", "10")

	list, total, err := svc.TagManagementGetList(page, limit)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}

	resp := md2.GetTagListResp{
		List: list,
		Paginate: md2.Paginate{
			Limit: utils.StrToInt(limit),
			Page:  utils.StrToInt(page),
			Total: total,
		},
	}
	e.OutSuc(c, resp, nil)
}

// AddTag
// @Summary      制度中心-会员中心-标签管理(新增)
// @Tags         会员中心
// @Description  标签管理(新增)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	 md.AddTagReq 	true  "新增的标签信息"
// @Success      200   {int}     	"新增的标签 ID"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/memberCenter/tagManagement/addTag [post]
func AddTag(c *gin.Context) {
	var req *md2.AddTagReq
	if err := c.ShouldBindJSON(&req); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err.Error())
		return
	}

	tag := model.UserTag{
		TagName:  req.Name,
		Memo:     req.Memo,
		CreateAt: time.Now().Format("2006-01-02 15:04:05"),
		IsPunish: req.IsPunish,
	}
	tagDb := implement.NewUserTagDb(db.Db)
	tagID, err := tagDb.UserTagInsert(&tag)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	e.OutSuc(c, tagID, nil)
}

// UpdateTag
// @Summary      制度中心-会员中心-标签管理(更新)
// @Tags         会员中心
// @Description  标签管理(更新)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	 md.UpdateTagReq 	true  "需要修改的标签信息"
// @Success      200   {int}     	"成功修改数据条数"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/memberCenter/tagManagement/updateTag [post]
func UpdateTag(c *gin.Context) {
	var req *md2.UpdateTagReq
	if err := c.ShouldBindJSON(&req); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err.Error())
		return
	}
	tagID := utils.StrToInt(req.TagID)
	tagDb := implement.NewUserTagDb(db.Db)
	oldTag, err := tagDb.UserTagGetOneByParams(map[string]interface{}{"key": "id", "value": tagID})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	if oldTag == nil {
		e.OutErr(c, e.ERR_NO_DATA, errors.New("标签不存在").Error())
		return
	}
	if oldTag.IsPunish != req.IsPunish {
		// 标签在被使用情况下不允许修改是否为处罚标签
		recordsDb := implement.NewUserTagRecordsDb(db.Db)
		exist, err := recordsDb.UserTagRecordsExist(0, tagID)
		if err != nil {
			e.OutErr(c, e.ERR_DB_ORM, err.Error())
			return
		}
		if exist {
			e.OutErr(c, e.ERR, errors.New("该标签已被使用,无法修改是否为处罚标签").Error())
			return
		}
	}
	tag := &model.UserTag{
		Id: tagID,
	}

	if req.Name != "" {
		tag.TagName = req.Name
	}

	if req.Memo != "" {
		tag.Memo = req.Memo
	}

	forceColumns := []string{"tag_name", "memo", "is_punish"}

	affected, err := tagDb.UserTagUpdate(tagID, tag, forceColumns...)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	e.OutSuc(c, affected, nil)
}

// DeleteTag
// @Summary      制度中心-会员中心-标签管理(删除)
// @Tags         会员中心
// @Description  标签管理(删除)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	 md.DeleteTagReq 	true  "需要删除的标签 ID"
// @Success      200   {int}     	"成功删除标签用户数"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/memberCenter/tagManagement/deleteTag [delete]
func DeleteTag(c *gin.Context) {
	var req *md2.DeleteTagReq
	if err := c.ShouldBindJSON(&req); err != nil {
		e.OutErr(c, e.ERR_INVALID_ARGS, err.Error())
		return
	}

	tagDb := implement.NewUserTagDb(db.Db)
	recordsDb := implement.NewUserTagRecordsDb(db.Db)
	session := db.Db.NewSession()
	defer session.Close()

	isUsed, err := recordsDb.UserTagRecordsIsUsedBySession(session, utils.StrToInt(req.TagID))
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	if isUsed {
		e.OutErr(c, e.ERR, errors.New("该标签已被使用,无法删除").Error())
		return
	}

	_, err = tagDb.UserTagDeleteBySession(session, req.TagID)
	if err != nil {
		session.Rollback()
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	affected, err := recordsDb.UserTagRecordsDeleteBySession(session, map[string]interface{}{
		"key":   "id",
		"value": req.TagID,
	})
	if err != nil {
		session.Rollback()
		e.OutErr(c, e.ERR_DB_ORM, err.Error())
		return
	}
	e.OutSuc(c, affected, nil)
}
